clear all
close all
clc
warning off

if matlabpool('size') == 0 ;
    matlabpool open normal;
end

disp('Program running...');

program_name = 'Table_1.m';

datafile = 'Canada_data.xlsx';

[num_data, txt_data] = xlsread(datafile,'Consumption');

year_name = num2cell(num_data(:,1));
country_name = txt_data(2:end)';

p_con_pc = num_data(:,2:end);

clear num_data txt_data

num_data = xlsread(datafile,'GDP');
p_gdp_pc = num_data(:,2:end);

clear num_data 

[T, N] = size(p_gdp_pc);

for ca_eq_ya = 0 : 1;
    
    % program_name should be the same as the name of this .m file
    if ca_eq_ya == 1;
        out_name = 'Table_1_gdp';
    elseif ca_eq_ya == 0;
        out_name = 'Table_1_con';
    end
    
    out_file = [out_name '.xls'];
    
    xlswrite(out_file, {['The result is created using datafile ' datafile ' and program ' program_name]}, 'Note');
        
    if ca_eq_ya == 1
        xlswrite(out_file, {'Results are based on aggregate output'}, 'Note', 'A3');
    else xlswrite(out_file, {'Results are based on aggregate consumption'}, 'Note', 'A3');
    end
    
    %% Sample period defines here
    start_date = [1;19;1];
    end_date = [18;37;37];
    
    %%
    N_bs = 1000;
    restrict = 1;
    restrict_cru = 1;
    no_mu = 0;
    
    cap_para = 1000;
    para_replace = 0.001;
    check_l = 0;
    f_min_unc = 1;
    
    max_rep = 200;
    max_rep_bs = 9;
    
    MM = length(start_date);
    
    rand_seed = 2;
    multi_c = 1;
    
    min_lambda_p = 0.001;
    max_lambda_p = 0.999;
    
    %% Optimization option
    diagnostic = 'off';
    disp_setting = 'off';
    DerivativeCheck = 'off';
    MaxFunEvals = 10000000;
    MaxIter = 10000000;
    tol = 1e-11;
    
    fmin_op = optimset('Display',disp_setting,...
        'MaxFunEvals', MaxFunEvals, 'MaxIter', MaxIter, 'FinDiffType', 'central', 'TolFun', tol, 'TolX', tol,...
        'Diagnostics', diagnostic);
    
    for loop = 1 : MM;
        sheet_name = [num2str(year_name{start_date(loop,1),1}) '_' num2str(year_name{end_date(loop,1),1})];
        
        temp_lambda_0 = NaN(N, max_rep);
        la_cu = NaN(N,1);
        
        para_star_temp = NaN(N,max_rep);
        LL = NaN(max_rep,1);
        
        rng(rand_seed);
        
        c = p_con_pc(start_date(loop,1):end_date(loop,1),1:N)*multi_c;
        y = p_gdp_pc(start_date(loop,1):end_date(loop,1),1:N)*multi_c;
        
        if ca_eq_ya == 0
            ca = mean(c,2);
        else
            ca = mean(y,2);
        end
        [T, N] = size(c);
        
        if no_mu == 1;
            c = c - ones(T,1)*mean(c);
            ca = ca - mean(ca);
            y = y - ones(T,1)*mean(y);
        end
        
        %% Crucini (1999)
        min_lambda = min_lambda_p + zeros(N,1);
        max_lambda = max_lambda_p*ones(N,1);
        
        cru_init = randn(N,max_rep);
        parfor i_loop = 1 : max_rep
            warning off
            [temp_lambda_0(:,i_loop), la_cu(i_loop,1)] = new_crucini_fminunc_cru(cru_init(:,i_loop), c, y, ca, restrict_cru, no_mu, min_lambda, max_lambda, cap_para, fmin_op, f_min_unc);
        end
        
        [~, tt_i] = min(la_cu);
        
        [~, ~, lambda_0, mu_0, sigma_0] = log_likelihood_crucini_est(temp_lambda_0(:,tt_i), c, y, ca, restrict_cru, no_mu, min_lambda, max_lambda, cap_para);
        
        %%
        if restrict == 1
            para_0 = [max(-cap_para, min(cap_para, log(1./lambda_0-1))) randn(N,max_rep-1)];
        else
            para_0 = [lambda_0 ones(N,1)*(0.1:0.1:0.9) rand(N,max_rep-10)];
        end
        
        %% Estimation
        parfor init_loop = 1 :  max_rep
            warning off
            [para_star_temp(:,init_loop), LL(init_loop,1)] ...
                = new_crucini_fminunc(para_0(:,init_loop), c , y, restrict, no_mu, check_l, min_lambda, max_lambda, cap_para, fmin_op, f_min_unc);
        end
        [~, LL_i] = min(LL);
        para_star = para_star_temp(:,LL_i);
        
        %% Numerical Check
        [~, para_out, lambda_star, sigma_star, mu_star, ~, x_star] = log_likelihood_main(para_star, c, y, restrict, no_mu, check_l, min_lambda, max_lambda, cap_para);
        
        [L_star, l_star, R_sq_star, r_sq_star, hess_stderr_star, outer_stderr_star, robust_stderr_star, grad_star, hess_star] = ...
            log_likelihood_unrestrict(para_out, c , y, no_mu, 0);
        
        %% Calculate the bias
        d = 1/N - lambda_star/sum(lambda_star);
        eta = lambda_star/sum(lambda_star);
        
        lh = NaN(N,1);
        lh_2 = NaN(N,1);
        for i = 1 : N
            temp_cov_1 = cov(ca - y(:,i), y*d);
            lh(i,1) = lambda_star(i,1)*( - (temp_cov_1(2,1))/var(ca - y(:,i)));
            lh_2(i,1) = (1 - (temp_cov_1(2,1))/var(ca-y(:,i)));
        end
        
        t_bias = mean(lambda_0 - lambda_star) - mean(lh);
        
        %% Crucini
        [L_crucini, l_crucini, R_sq_cru, r_sq_cru, stderr_cru_h, stderr_cru_o, stderr_cru_r, grad_crucini, hess_crucini] = ...
            log_likelihood_crucini([lambda_0; sigma_0; mu_0], c , y, ca, no_mu);
        
        %% Resampling
        lh_bs = NaN(N, N_bs);
        lh_bs_2 = NaN(N, N_bs);
        lambda_0_bs = NaN(N, N_bs);
        lambda_bs = lambda_0_bs;
        m_lh_bs = NaN(N_bs,1);
        m_lh_bs_2 = m_lh_bs;
        m_rh_bs = NaN(N_bs,1);
        bias = NaN(N_bs,1);
        
        for i_bs = 1 : N_bs;
            x_bs = NaN(T,N);
            for i = 1 : N;
                x_bs(:,i) = x_star(randi(T,[T,1]),i);
            end
            
            % Regenerate the consumption
            c_bs = y + ones(T,1)*mu_star' + ((ones(T,1)*lambda_star').*( - y + ((y*eta)*ones(1,N)))) + x_bs.*(ones(T,1)*sigma_star');
            
            if ca_eq_ya == 0
                ca_bs = mean(c_bs,2);
            else
                ca_bs = mean(y,2);
            end
            
            cru_init = randn(N,max_rep_bs);
            parfor i_loop = 1 : max_rep_bs
                warning off
                [temp_lambda_0(:,i_loop), la_cu(i_loop,1)] = new_crucini_fminunc_cru(cru_init(:,i_loop), c_bs, y, ca_bs, restrict_cru, no_mu, min_lambda, max_lambda, cap_para, fmin_op, f_min_unc);
            end
            
            [~, tt_i] = min(la_cu);
            
            [~, ~, lambda_0_bs(:,i_bs)] = log_likelihood_crucini_est(temp_lambda_0(:,tt_i), c_bs, y, ca_bs, restrict_cru, no_mu, min_lambda, max_lambda, cap_para);
            
            %%
            if restrict == 1
                para_0_bs = [max(-cap_para, min(cap_para, log(1./lambda_0_bs(:,i_bs)-1))) randn(N,max_rep_bs-1)];
            else
                para_0_bs = [lambda_0_bs(:,i_bs) ones(N,1)*(0.1:0.1:0.9) rand(N,max_rep_bs-10)];
            end
            
            %% Estimation
            parfor init_loop = 1 :  max_rep_bs
                warning off
                [para_star_temp_bs(:,init_loop), LL_bs(init_loop,1)] ...
                    = new_crucini_fminunc(para_0_bs(:,init_loop), c_bs, y, restrict, no_mu, check_l, min_lambda, max_lambda, cap_para, fmin_op, f_min_unc);
            end
            [~, LL_i_bs] = min(LL_bs);
            
            [~, ~, lambda_bs(:,i_bs)] = log_likelihood_main(para_star_temp_bs(:,LL_i_bs), c_bs, y, restrict, no_mu, check_l, min_lambda, max_lambda, cap_para);
            
            %% Calculate the bias
            d = 1/N - lambda_bs(:,i_bs)/sum(lambda_bs(:,i_bs));
            
            for i = 1 : N
                temp_cov_1 = cov(ca_bs - y(:,i), y*d);
                lh_bs(i,i_bs) = lambda_bs(i,i_bs)*( - (temp_cov_1(2,1))/var(ca_bs - y(:,i)));
                lh_bs_2(i,i_bs) = (1 - (temp_cov_1(2,1))/var(ca_bs - y(:,i)));
            end
            
            m_lh_bs(i_bs, 1) = mean(lh_bs(:,i_bs));
            m_rh_bs(i_bs, 1) = mean(lambda_0_bs(1:N, i_bs) - lambda_bs(1:N, i_bs));
            bias(i_bs, 1) = m_rh_bs(i_bs,1) - m_lh_bs(i_bs,1);
            m_lh_bs_2(i_bs,1) = mean(lh_bs_2(:,i_bs));
        end
        
        %%
        if no_mu == 0
            row_name = [{'Country'} {'Unequal: Lambda'} {'(Standard Error)'}...
                {'Equal: Lambda'} {'(Standard Error)'} ...
                {'lambda(equal) - lambda(unequal)'}];
            result = [row_name; [ [country_name; {'Average'}]...
                num2cell([lambda_star; mean(lambda_star)]) num2cell([outer_stderr_star(1:N,1); mean(outer_stderr_star(1:N,1))])...
                num2cell([lambda_0; mean(lambda_0)]) num2cell([stderr_cru_o(1:N,1); mean(stderr_cru_o(1:N,1))])...
                num2cell([lambda_star(1:N) - lambda_0(1:N,1); mean(lambda_star(1:N) - lambda_0(1:N,1))]) ] ];
        else
            row_name = [{'Country'} {'Unequal: Lambda'} {'(Standard Error)'} ...
                {'Equal: Lambda'} {'(Standard Error)'} ...
                {'lambda(equal) - lambda(unequal)'}];
            result = [row_name; [ [country_name; {'Average'}]...
                num2cell([lambda_star; mean(lambda_star)]) num2cell([outer_stderr_star(1:N,1); mean(outer_stderr_star(1:N,1))])...
                num2cell([lambda_0; mean(lambda_0)]) num2cell([stderr_cru_o(1:N,1); mean(stderr_cru_o(1:N,1))])...
                num2cell([lambda_star(1:N) - lambda_0(1:N,1); mean(lambda_star(1:N) - lambda_0(1:N,1))]) ] ];
        end
        
        disp(result);
        disp(['Unequal overall R-sq: ' num2str(R_sq_star)]);
        disp(['Equal overall R-sq: ' num2str(R_sq_cru)]);save(out_name);
        xlswrite(out_file, result, sheet_name);
       
        xlswrite(out_file, [[{'  '};{'Mean'};{'5%-tile'};{'95%-tile'}] [[{'Actual bias'} {'Implied bias'}]; num2cell([[mean(lambda_0 - lambda_star) mean(lh)]; [prctile(m_rh_bs,[5;95]) prctile(m_lh_bs,[5;95])]])] ],...
            [sheet_name '_bs'], 'A1');
        xlswrite(out_file, [{'Proportion of Equal'}; num2cell(mean(1*(m_lh_bs .* m_rh_bs > 0)))], [sheet_name '_bs'], 'D1');
        xlswrite(out_file, [{'# of BS-sample'}; num2cell(N_bs)], [sheet_name '_bs'], 'E1');
    end
    save(out_name);
    winopen(out_file);
end