function [L, l, R_sq, r_sq, hess_stderr, outer_stderr, robust_stderr, grad, hess] = log_likelihood_crucini(para, c , y, ca, no_mu)
% version: 2013/Jan/9 6:31 PM NY
[T, N] = size(c);

lambda = para(1:N,1);
sigma = para(N+1:2*N,1);

if no_mu == 1;
    mu = zeros(N,1);
else mu = para(2*N+1:3*N,1);
end

x = ((c - y - ones(T,1)*mu' -((ones(T,1)*lambda').*(ca*ones(1,N) - y)))./(ones(T,1)*sigma'));

L = -sum(sum(x.^2))/2 - T*sum(log(sigma)) - (N*T*log(2*pi))/2;

l = -sum(x.^2)/2 - T*log(sigma') - T*log(2*pi)/2;
l = l';

xx = (c - y - ones(T,1)*mu' -((ones(T,1)*lambda').*(ca*ones(1,N) - y)));

R_sq = (1 - sum(sum(xx.^2))./sum(sum(c-y).^2))';
r_sq = (1 - sum(xx.^2)./sum((c-y).^2))';

%% First derivative of L
% Derivative w.r.t lambda
dL_dlambda = (sum(x.*(ca*ones(1,N) - y)./(ones(T,1)*sigma')))';

dL_dsigma = ((sum((x.^2)) - T)./(sigma'))';

if no_mu == 0
    % Derivative w.r.t mu
    dL_dmu = (sum(x)./sigma')';
    grad = [dL_dlambda; dL_dsigma; dL_dmu];
else grad = [dL_dlambda; dL_dsigma];
end

%%
dl_dpara = zeros((3-no_mu)*N,1);
outer_prod = zeros((3-no_mu)*N, (3-no_mu)*N);

if no_mu == 0
    for i1 = 1 : N;
        for t2 = 1 : T;
            for p3 = 1 : N;
                dl_dpara(p3,1) = (((c(t2,i1) - y(t2,i1) - mu(i1,1) - lambda(i1,1)*(ca(t2,1) - y(t2,i1)))/sigma(i1,1))*(ca(t2,1)-y(t2,i1))/sigma(i1,1)*(i1 == p3));
                dl_dpara(p3+N,1) = (((c(t2,i1) - y(t2,i1) - lambda(i1,1)*(ca(t2,1) - y(t2,i1)))^2 - sigma(i1,1)^2)/(sigma(i1,1)^3))*(i1 == p3);
                dl_dpara(p3+2*N,1) = ((c(t2,i1) - y(t2,i1) - mu(i1,1) - lambda(i1,1)*(ca(t2,1) - y(t2,i1)))/(sigma(p3,1)^2))*(i1 == p3);
            end
            outer_prod = outer_prod + dl_dpara*dl_dpara';
        end
    end
else
    for i1 = 1 : N;
        for t2 = 1 : T;
            for p3 = 1 : N;
                dl_dpara(p3,1) = (((c(t2,i1) - y(t2,i1) - mu(i1,1) - lambda(i1,1)*(ca(t2,1) - y(t2,i1)))/sigma(i1,1))*(ca(t2,1)-y(t2,i1))/sigma(i1,1)*(i1 == p3));
                dl_dpara(p3+N,1) = (((c(t2,i1) - y(t2,i1) - lambda(i1,1)*(ca(t2,1) - y(t2,i1)))^2 - sigma(i1,1)^2)/(sigma(i1,1)^3))*(i1 == p3);
            end
            outer_prod = outer_prod + dl_dpara*dl_dpara';
        end
    end
end

%% Second-order derivative
d2L_dlambda2 = -diag(sum(((ca*ones(1,N) - y)./(ones(T,1)*sigma')).^2));
d2L_dsigmadlambda = diag(-2*sum((c - y - ones(T,1)*mu' - (ones(T,1)*lambda').*(ca*ones(1,N) - y)).*(ca*ones(1,N) - y))./(sigma'.^3));
d2L_dsigma2 = diag((T - 3*sum((c - y - ones(T,1)*mu' - (ones(T,1)*lambda').*(ca*ones(1,N) - y)).^2)./(sigma'.^2))./(sigma'.^2));

if no_mu == 0
    d2L_dmu2 = diag(-T./(sigma.^2));
    d2L_dmudsigma = -diag(sum(ca*ones(1,N) - y)./(sigma'.^2));
    d2L_dmudlambda = -diag(sum((ca*ones(1,N) - y))./(sigma'.^2));
    hess = [d2L_dlambda2 d2L_dsigmadlambda' d2L_dmudlambda';
        d2L_dsigmadlambda d2L_dsigma2 d2L_dmudsigma';
        d2L_dmudlambda d2L_dmudsigma d2L_dmu2];
else
    hess = [d2L_dlambda2 d2L_dsigmadlambda';
        d2L_dsigmadlambda d2L_dsigma2];
end

%% Calculate the standard error of estimates
hess_stderr = sqrt(diag(inv(-hess)));
outer_stderr = sqrt(diag(inv(outer_prod)));
robust_stderr = sqrt(diag((((-hess)\(outer_prod))/(-hess))));

for jj = 1 : (3-no_mu)*N;
    if ~isreal(hess_stderr(jj,1))
        hess_stderr(jj,1) = NaN;
    end
    if ~isreal(outer_stderr(jj,1))
        outer_stderr(jj,1) = NaN;
    end
    if ~isreal(robust_stderr(jj,1))
        robust_stderr(jj,1) = NaN;
    end
end

end
