function [L, l, R_sq, r_sq, hess_stderr, outer_stderr, robust_stderr, grad, hess, grad_check, hess_check, x] ...
    = log_likelihood_unrestrict(para, c , y, no_mu, check_d)
% This is the joint log-likelihood function
% version: 2013/Jan/08 11:28 PM NY
[T, N] = size(c);

lambda = para(1:N,:);

sigma = para(N+1:2*N,:);

M = size(lambda,2);

if no_mu == 1;
    mu = zeros(N,M);
    grad = NaN(2*N,M);
    hess = NaN(2*N,2*N,M);
    outer_prod = zeros(2*N, 2*N, M);
else mu = para(2*N+1:3*N,:);
    grad = NaN(3*N,M);
    hess = NaN(3*N,3*N,M);
    outer_prod = zeros(3*N, 3*N, M);
end

hess_stderr = grad;
outer_stderr = hess_stderr;
robust_stderr = hess_stderr;

L = NaN(M,1);
l = NaN(N,M);
R_sq = NaN(M,1);
r_sq = NaN(N,M);

for ii = 1 : M
    %%
    d = lambda(:,ii)/sum(lambda(:,ii));
    
    sp1 = c - y - ones(T,1)*mu(:,ii)' - ((ones(T,1)*lambda(:,ii)').*( - y + ((y*d)*ones(1,N))));
    
    x = sp1./(ones(T,1)*(sigma(:,ii)'));
    L(ii,1) = -sum(sum(x.^2))/2 - T*sum(log(sigma(:,ii))) - (N*T*log(2*pi))/2;
    
    l(:,ii) = (-sum(x.^2)/2 - T*log(sigma(:,ii)') - T*log(2*pi)/2)';
    
    R_sq(ii,1) = (1 - sum(sum(sp1.^2))./sum(sum((c-y).^2)))';
    r_sq(:,ii) = (1 - sum(sp1.^2)./sum((c-y).^2))';
    
    %% First derivative of L
    % Derivative w.r.t lambda
    dL_dlambda = zeros(N,1);
    
    for j = 1 : N;
        z = ((y(:,j)*ones(1,N) - y)*lambda(:,ii))/(sum(lambda(:,ii)))^2;
        dL_dlambda(j,1) = sum(sum(x.*(ones(T,1)*(lambda(:,ii)./sigma(:,ii))').*(z*ones(1,N))))...
            + sum(x(:,j).*((- y(:,j) + y*d)/sigma(j,ii)));
    end
    
    % Derivative w.r.t sigma
    dL_dsigma_sq_1 = zeros(T,N);
    dL_dsigma_1 = sum(sp1.^2)./(sigma(:,ii)'.^2);
    %dL_dsigma_sq = (dL_dsigma_1 - T)./(2*(sigma'.^2));
    %dL_dsigma_sq = dL_dsigma_sq';
    dL_dsigma = (dL_dsigma_1 - T)./(sigma(:,ii)');
    dL_dsigma = dL_dsigma';
    
    if no_mu == 0
        % Derivative w.r.t mu
        dL_dmu = sum(sp1)./(sigma(:,ii)'.^2);
        dL_dmu = dL_dmu';
        grad(:,ii) = [dL_dlambda; dL_dsigma; dL_dmu];
    else grad(:,ii) = [dL_dlambda; dL_dsigma];
    end
    
    %%
    dl_dpara = zeros((3-no_mu)*N,1);
    
    if no_mu == 0
        for i1 = 1 : N;
            for t2 = 1 : T;
                for p3 = 1 : N;
                    dl_dpara(p3,1) = x(t2,i1)*((lambda(i1,ii)*((y(t2,p3) - y(t2,:))*lambda(:,ii))/(sigma(i1,ii)*(sum(lambda(:,ii))^2))) + ((y(t2,:)*d-y(t2,i1))/sigma(i1,ii))*(i1 == p3));
                    dl_dpara(p3+N,1) = (((c(t2,p3) - y(t2,p3) - mu(p3,1) - lambda(p3,ii)*sum(y(t2,:)*d - y(t2,p3)))^2 - sigma(p3,ii)^2)/sigma(p3,ii)^3)*(i1 == p3);
                    dl_dpara(p3+2*N,1) = ((c(t2,p3) - y(t2,p3) - lambda(p3,ii)*(y(t2,:)*d - y(t2,p3)))/(sigma(p3,ii)^2))*(i1 == p3);
                end
                outer_prod(:,:,ii) = outer_prod(:,:,ii) + dl_dpara*dl_dpara';
            end
        end
    else
        for i1 = 1 : N;
            for t2 = 1 : T;
                for p3 = 1 : N;
                    dl_dpara(p3,1) = x(t2,i1)*((lambda(i1,ii)*((y(t2,p3) - y(t2,:))*lambda(:,ii))/(sigma(i1,ii)*(sum(lambda(:,ii))^2))) + ((y(t2,:)*d-y(t2,i1))/sigma(i1,ii))*(i1 == p3));
                    dl_dpara(p3+N,1) = (((c(t2,p3) - y(t2,p3) - mu(p3,1) - lambda(p3,ii)*sum(y(t2,:)*d - y(t2,p3)))^2 - sigma(p3,ii)^2)/sigma(p3,ii)^3)*(i1 == p3);
                end
                outer_prod(:,:,ii) = outer_prod(:,:,ii) + dl_dpara*dl_dpara';
            end
        end
    end

    %% Second-order derivative
    dlam_dlam = diag(ones(N,1));
    
    d2L_dlambda2 = zeros(N,N);
    d2L_dsigmadlambda = zeros(N,N);
    d2L_dsigma2 = diag((T - 3*sum(sp1.^2)'./(sigma(:,ii).^2))./(sigma(:,ii).^2));
    
    d2L_dmudlambda = zeros(N,N);
    
    for h = 1 : N;
        for k = 1 : N;
            A = (sp1./(ones(T,1)*sigma(:,ii)'))...
                .*(((ones(T,1)*dlam_dlam(k,:)).*(((y(:,h)*ones(1,N) - y)*(lambda(:,ii)/(sum(lambda(:,ii))^2)))*ones(1,N))./(ones(T,1)*sigma(:,ii)'))...
                + ((ones(T,1)*dlam_dlam(h,:)).*(((y(:,k)*ones(1,N) - y)*(lambda(:,ii)/(sum(lambda(:,ii))^2)))*ones(1,N))./(ones(T,1)*sigma(:,ii)'))...
                + ((ones(T,1)*(lambda(:,ii)'./sigma(:,ii)')).*(((y(:,h)-y(:,k))*ones(1,N))/(sum(lambda(:,ii))^2) - 2*(((y(:,h)*ones(1,N)-y)*lambda(:,ii))*ones(1,N))/(sum(lambda(:,ii))^3))));
            
            B = (((((y - y(:,k)*ones(1,N))*(lambda(:,ii)/(sum(lambda(:,ii))^2)))*ones(1,N)).*(ones(T,1)*(lambda(:,ii)./sigma(:,ii))'))...
                - (( - y + (y*d)*ones(1,N))./(ones(T,1)*sigma(:,ii)')).*(ones(T,1)*dlam_dlam(k,:)))...
                .*(((((y(:,h)*ones(1,N) - y)*(lambda(:,ii)/(sum(lambda(:,ii))^2)))*ones(1,N)).*(ones(T,1)*(lambda(:,ii)./sigma(:,ii))'))...
                + (( - y + (y*d)*ones(1,N))./(ones(T,1)*sigma(:,ii)')).*(ones(T,1)*dlam_dlam(h,:)));...
                
            d2L_dlambda2(k,h) = sum(sum(A + B));
        end
        
        d2L_dsigmadlambda(:,h) = 2*sum(sp1.*((((y - y(:,h)*ones(1,N))*(lambda(:,ii)/(sum(lambda(:,ii))^2)))*lambda(:,ii)') ...
            - (ones(T,1)*dlam_dlam(h,:)).*(( - y + (y*d)*ones(1,N)))))'./(sigma(:,ii).^3);
        
        if no_mu == 0
            d2L_dmudlambda(:,h) = sum((((y - y(:,h)*ones(1,N))*(lambda(:,ii)/(sum(lambda(:,ii))^2)))*(lambda(:,ii)./(sigma(:,ii).^2))') ...
                - (ones(T,1)*dlam_dlam(h,:)).*(( - y + (y*d)*ones(1,N))./(ones(T,1)*(sigma(:,ii)'.^2))))';
        end
    end
    
    if no_mu == 0
        d2L_dmu2 = diag(-T./(sigma(:,ii).^2));
        d2L_dmudsigma = diag(2*(T*mu(:,ii) - sum(c - y -((ones(T,1)*lambda(:,ii)').*( - y + ((y*d)*ones(1,N)))))')./(sigma(:,ii).^3));
        hess(:,:,ii) = [d2L_dlambda2 d2L_dsigmadlambda' d2L_dmudlambda';
            d2L_dsigmadlambda d2L_dsigma2 d2L_dmudsigma';
            d2L_dmudlambda d2L_dmudsigma d2L_dmu2];
    else
        hess(:,:,ii) = [d2L_dlambda2 d2L_dsigmadlambda';
            d2L_dsigmadlambda d2L_dsigma2];
    end
    
    %% Calculate the standard error of estimates
    hess_stderr(:,ii) = sqrt(diag(inv(-hess(:,:,ii))));
    outer_stderr(:,ii) = sqrt(diag(inv(outer_prod(:,:,ii))));    
    robust_stderr(:,ii) = sqrt(diag((((-hess(:,:,ii))\(outer_prod(:,:,ii)))/(-hess(:,:,ii)))));
    
    for jj = 1 : (3-no_mu)*N;
       if ~isreal(hess_stderr(jj,ii)) 
           hess_stderr(jj,ii) = NaN;
       end
       if ~isreal(outer_stderr(jj,ii)) 
           outer_stderr(jj,ii) = NaN;
       end
       if ~isreal(robust_stderr(jj,ii)) 
           robust_stderr(jj,ii) = NaN;
       end
    end
    
    %%
    if check_d == 1
        
        grad_check = grad;
        hess_check = hess;
        sum_d = y*d;
        
        dL_dlambda_check = zeros(N,1);
        d2L_dlambda2_check = zeros(N,N);
        term1 = zeros(T,N);
        term2 = zeros(T,N);
        
        for h = 1 : N
            for i = 1 : N;
                for t = 1 : T;
                    term1(t,i) = ((c(t,i) - y(t,i) - mu(i,ii) - lambda(i,ii)*( - y(t,i) + sum_d(t,1)))/sigma(i,ii))...
                        *(lambda(i,ii)*((y(t,h) - y(t,:))*lambda(:,ii))/(sigma(i,ii)*(sum(lambda(:,ii))^2))...
                        + dlam_dlam(i,h)*( - y(t,i) + sum_d(t,1))/sigma(i,ii));
                end
            end
            dL_dlambda_check(h,1) = sum(sum(term1));
        end
        
        dL_dmu_check = zeros(N,1);
        dL_dsigma_sq_check = zeros(N,1);
        dL_dsigma_check = zeros(N,1);
        
        for k = 1 : N;
            for t = 1 : T
                dL_dmu_check(k,1) = dL_dmu_check(k,1) + (c(t,k) - y(t,k) - mu(k,ii) - lambda(k,ii)*( - y(t,k) + sum_d(t,1)));
                dL_dsigma_sq_1(k,1) = dL_dsigma_sq_1(k,1) + ((c(t,k) - y(t,k) - mu(k,ii) - lambda(k,ii)*( - y(t,k) + sum_d(t,1)))^2)/(sigma(k,ii)^2);
            end
            dL_dmu_check(k,1) = dL_dmu_check(k,1)/(sigma(k,ii)^2);
            dL_dsigma_sq_check(k,1) = (dL_dsigma_sq_1(k,1) - T)/(2*(sigma(k,ii)^2));
            dL_dsigma_check(k,1) = (dL_dsigma_sq_1(k,1) - T)/(sigma(k,ii));
        end
        
        if no_mu == 0
            grad_check(:,ii) = [dL_dlambda_check; dL_dsigma_check; dL_dmu_check];
        else grad_check(:,ii) = [dL_dlambda_check; dL_dsigma_check];
        end
        
        %Second-order derivative
        for h = 1 : N;
            for k = 1 : N;
                for i = 1 : N
                    for t = 1 : T;
                        term2(t,i) = ((lambda(i,ii)*((y(t,:) - y(t,k))*lambda(:,ii))/(sigma(i,ii)*(sum(lambda(:,ii))^2)) ...
                            - dlam_dlam(i,k)*( - y(t,i) + sum_d(t,1))/sigma(i,ii))...
                            * (lambda(i,ii)*((y(t,h) - y(t,:))*lambda(:,ii))/(sigma(i,ii)*(sum(lambda(:,ii))^2)) ...
                            + dlam_dlam(i,h)*( - y(t,i) + sum_d(t,1))/sigma(i,ii)))...
                            + ((c(t,i) - y(t,i) - mu(i,ii) - lambda(i,ii)*( - y(t,i) + sum_d(t,1)))/sigma(i,ii)) ...
                            * (dlam_dlam(i,k) * (((y(t,h) - y(t,:))*lambda(:,ii)) /(sigma(i,ii)*(sum(lambda(:,ii))^2))) ...
                            + dlam_dlam(i,h) * (((y(t,k) - y(t,:))*lambda(:,ii)) /(sigma(i,ii)*(sum(lambda(:,ii))^2))) ...
                            + (lambda(i,ii)/sigma(i,ii))*((y(t,h) - y(t,k))/(sum(lambda(:,ii))^2) - (2*((y(t,h) - y(t,:))*lambda(:,ii))/(sum(lambda(:,ii))^3))));
                    end
                end
                d2L_dlambda2_check(k,h) = sum(sum(term2));
            end
        end
        hess_check(:,:,ii) = d2L_dlambda2_check;
    end
end
end % end of main program
